import java.awt.*;
import java.awt.image.CropImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.FilteredImageSource;

/**
* Ist ein Canvas, der ein angegebenes Image als eine Animation anzeigt.
* Das Image enthlt eine Folge von Bildern nebeneinander, die alle die gleichen
* Abmessungen haben mssen.
* Version 1.0 25.11.1997
* @version 1.1 26.11.1997 (erweiterter Construktor, Reset)
* @author Chris Hbsch
*/
public class AnimCanvas extends Canvas implements Runnable{
  Image img;
  Image images[];
  Thread runner=null;
  int breite=34, hoehe=34, anzahl=12;
  int aktImage=0;
  long sleepTime=250;
  Canvas canvas;
  
  /**
  * Erzeugt einen neues AnimCanvas. Fr die Gre eines Einzelbildes werden 34x34 Pixel 
  * angenommen und es werden 12 Bilder im Abstand von jeweils 250 ms angezeigt.
  * @param img das Bild, da die Einzelbilder enthlt
  */
  public AnimCanvas(Image img) {
    this.img=img;
    MediaTracker mt=new MediaTracker(this);
    mt.addImage(img,0);
    try {
      mt.waitForID(0);
    } catch (InterruptedException e) {
    };
    hoehe=34; breite=34; anzahl=12;
    images=new Image[anzahl];
    for (int i=0;i<anzahl;i++) {
      CropImageFilter filter = new CropImageFilter(breite*i,0,breite,hoehe);
      ImageProducer producer = new FilteredImageSource(img.getSource(), filter);
      images[i]=createImage(producer);
    }
  }

  /**
  * Erzeugt einen neues AnimCanvas. Fr die Gre eines Einzelbildes werden <tt>x</tt>x<tt>y</tt> Pixel 
  * angenommen und es werden <tt>n</tt> Bilder im Abstand von jeweils <tt>t</tt> ms angezeigt.
  * @param img das Bild, da die Einzelbilder enthlt
  * @param x breite des Bildes
  * @param y hoehe des Bildes
  * @param n anzahl der Bilder
  * @param t Wartezeit zwischen den Bildern in ms
  */
  public AnimCanvas(Image img, int x, int y, int n, int t) {
    this.img=img;
    MediaTracker mt=new MediaTracker(this);
    mt.addImage(img,0);
    try {
      mt.waitForID(0);
    } catch (InterruptedException e) {
    };
    hoehe=y; breite=x; anzahl=n; sleepTime=t;
    images=new Image[anzahl];
    for (int i=0;i<anzahl;i++) {
      CropImageFilter filter = new CropImageFilter(breite*i,0,breite,hoehe);
      ImageProducer producer = new FilteredImageSource(img.getSource(), filter);
      images[i]=createImage(producer);
    }
  }


  public Dimension preferredSize() {
    return new Dimension(breite, hoehe);
  }
  
  /**
  * Ruft sofort paint auf, um flackern zu vermeiden.
  */
  public void update(Graphics g) {
    paint(g);
  }
  
  /**
  * Zeichnet den aktuellen Teil des Images in die Mitte des Fensters.
  */
  public void paint(Graphics g) {
    Dimension d=size();
    int x=(d.width-breite)/2;
    int y=(d.height-hoehe)/2;
    g.drawImage(images[aktImage], x, y, this);
  }
  
  /**
  * Animationsschleife nach <tt>Runnable</tt> Interface.
  */
  public void run() {
    while (true) {
      aktImage=(aktImage+1)%anzahl;
      repaint();
      try {
        runner.sleep(sleepTime);
      } catch (InterruptedException e) {
      }  
    }
  }
  
  /**
  * Stopt die Animation.
  */
  public void stop() {
    if (runner != null) {
      runner.stop();
      runner=null;
    }
  }    
  
  /**
  * Startet die Animation.
  */
  public void start() {
    if (runner == null) {
      runner=new Thread(this);
      runner.start();
    }
  }
  
  /**
  * Zeigt das erste Bild an.
  */
  public void reset() {
    aktImage=0;
    repaint();
  }
  
}
